# -*- coding: utf-8 -*-

"""Tools dedicated to molecules"""


# ----------------------- Imports --------------------------------
import os
import subprocess
from pathlib import Path
# import rdkit
#from openbabel import pybel
# My modules
import config


#--------------------------Functions------------------------------
# Setup Open Babel
if os.name == 'posix':
    obabel_exec = Path(config.BABEL_PATH).joinpath("obabel")
    shell_choice = False
elif os.name == 'nt':
    obabel_exec = Path(config.BABEL_PATH).joinpath("obabel.exe")
    shell_choice = True


def smi_OK(fichier_smi):
    """
    Check that there is no ID in the smile file. Else, remove it and rename original file in .old
    Check that there is no salt. Else, remove it with rdkit.
    """
    chemin = fichier_smi.parent
    smi_OK = "" #fichier avec smiles OK (sans ID, sans sel)
    lignes = []
    with open(fichier_smi, 'r') as smif:
        lignes = [l for l in smif.readlines()]
    if len(lignes) == 1:
        if len(lignes[0].split()) == 1:    # i.e. smi OK
            smi_OK = fichier_smi
        else:
            smi_str = lignes[0].split()[0] # on copie le premier élément, enlève l'ID
            smi_OK = chemin.joinpath(fichier_smi.stem + "_noID.smi")
            with open(smi_OK, 'w') as new_smif:
                new_smif.write(smi_str)
            if Path(str(fichier_smi) + ".old").exists():    # cas où erreur conversion pas allée jusqu'au cif
                os.remove(str(fichier_smi) + ".old")
            os.rename(str(fichier_smi), str(fichier_smi) + ".old")
        if len(smi_str.split('.')) > 1:    # i.e. il y a des sels
            # enlever les sels
            # couper là où '.' et garder le plus gros
            smi_str_no_salt = ""
            for part in smi_str.split('.'):
                if len(part) > len (smi_str_no_salt):
                    smi_str_no_salt = part
            #mol = rdkit.Chem.MolFromSmiles(smi_str)
            #mol_nosalt = rdkit.remover.StripMol(mol)
            smi_noID_nosalt = chemin.joinpath(smi_OK.stem + "_nosalt.smi")
            with open(smi_noID_nosalt, 'w') as new_smif:
                #new_smif.write(rdkit.Chem.MolToSmiles(mol_nosalt))
                new_smif.write(smi_str_no_salt)
            if Path(str(smi_OK) + ".old").exists():    # cas où erreur conversion pas allée jusqu'au cif
                os.remove(str(smi_OK) + ".old")
            os.rename(str(smi_OK), str(smi_OK) + ".old")
            smi_OK = smi_noID_nosalt
            print("Salt stripped ...")
    else:   # fichier smi vide
        print("smiles file " + fichier_smi.name + " empty ...")
    return smi_OK


def smi_file_cano(fichier_smi):
    """
    Convert smiles file in canonized smiles file with Open Babel (else Elbow can fail). Rename original file in .old
    """
    smi_OK_cano = ""
    if fichier_smi: # cas où pas d'erreur lors de l'étape d'enlever les sels
        chemin = fichier_smi.parent
        smi_OK_cano = chemin.joinpath(fichier_smi.stem + "_cano.smi")
        #with open(fichier_smi, 'r') as smif:
            #smi_str = smif.readlines()[0]
        #smi_str_cano = pybel.readstring("smi", smi_str).write("can").split()[0]
        #mol = rdkit.Chem.MolFromSmiles(smi_str)
        #mol_cano = rdkit.Chem.MolToSmiles(mol, canonical=True)
        #with open(smi_OK_cano, 'w') as new_smif:
            #new_smif.write(rdkit.Chem.MolToSmiles(mol_cano))
            #new_smif.write(smi_str_cano)
        babel = subprocess.Popen([obabel_exec, str(fichier_smi), '-ocan', '-O', str(smi_OK_cano),
                                  ], stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=shell_choice, encoding='utf-8')
        out, err = babel.communicate()
        if babel.returncode != 0:
            #print(out, err, "Canonization failed ...")
            pass
        else:
            #print("Canonized ...")
            if Path(str(fichier_smi) + ".old").exists():    # cas où erreur conversion pas allée jusqu'au cif
                os.remove(str(fichier_smi) + ".old")
            os.rename(str(fichier_smi), str(fichier_smi) + ".old")
    return smi_OK_cano

def smi_cano(smi_str):
    """
    Convert smiles in canonized smiles with Open Babel (else Elbow can fail).
    :return: smiles string after canonization, code error, and error (tuple)
    """
    babel = subprocess.Popen([obabel_exec, '-:' + smi_str , '-ocan'],
                             stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=shell_choice, encoding='utf-8')
    out, err = babel.communicate()
    #print("out: ", out)
    #print("err: ", err)
    #if babel.returncode != 0:
        #print(out, err, "Canonization failed ...")
    #else:
        #print("Canonized ...")
    return out, babel.returncode, err